package com.example.android.iap;

import android.support.annotation.NonNull;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.SocketException;
import java.net.URL;

class HttpPostRunnable implements Runnable {
    private String mUrl;
    private JSONObject mBody;
    private HttpRequestListener mHttpRequestListener;

    public HttpPostRunnable(@NonNull String url,
                            JSONObject body,
                            @NonNull HttpRequestListener httpRequestListener) {
        mUrl = url;
        mBody = body;
        mHttpRequestListener = httpRequestListener;
    }

    @Override
    public void run() {
        HttpURLConnection connection = null;

        InputStream inputStream = null;

        try {
            URL postUrl = new URL(mUrl);
            connection = (HttpURLConnection) postUrl.openConnection();
            connection.setRequestMethod("POST");
            connection.setRequestProperty("Content-Type", "application/json");

            if (mBody != null) {
                connection.setDoOutput(true);
                OutputStream os = connection.getOutputStream();
                os.write(mBody.toString().getBytes());
                os.flush();
            }

            connection.connect();

            int responseCode = connection.getResponseCode();

            if (responseCode == HttpURLConnection.HTTP_OK) {
                inputStream = connection.getInputStream();
                String responseMessage = readStreamToString(inputStream);
                int resultCode = getResultCode(responseMessage);

                if (resultCode == 0) {
                    mHttpRequestListener.onSuccess();
                } else {
                    mHttpRequestListener.onError(resultCode, responseMessage);
                }
            } else {
                mHttpRequestListener.onError(responseCode, connection.getResponseMessage());
            }

        } catch (SocketException e) {
            mHttpRequestListener.onError(-1, e.getMessage());
        } catch (IOException e) {
            mHttpRequestListener.onError(-1, e.getMessage());
        } finally {
            if (connection != null) {
                connection.disconnect();
            }

            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private String readStreamToString(InputStream inputStream) throws IOException {
        StringBuilder stringBuilder = new StringBuilder();
        BufferedInputStream bufferedInputStream = null;
        InputStreamReader reader = null;

        try {
            bufferedInputStream = new BufferedInputStream(inputStream);
            reader = new InputStreamReader(bufferedInputStream);

            char[] buffer = new char[1024];

            int n;
            while ((n = reader.read(buffer)) != -1) {
                stringBuilder.append(buffer, 0, n);
            }
        } finally {
            if (bufferedInputStream != null) {
                bufferedInputStream.close();
            }
            if (reader != null) {
                reader.close();
            }
        }

        return stringBuilder.toString();
    }

    private int getResultCode(String responseMessage) {
        try {
            JSONObject responseJson = new JSONObject(responseMessage);
            if (responseJson.has("header")) {
                JSONObject header = responseJson.getJSONObject("header");
                if (header.has("resultCode")) {
                    return header.getInt("resultCode");
                }
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }

        return -1;
    }

    interface HttpRequestListener {
        void onSuccess();

        void onError(int code, String message);
    }
}
